$	SAV_VER = 'F$VERIFY(0)'
$! (Remember if SET VERIFY is turned on, but leave it off for now.)
$! **************************************************************************** !$
$!										!$
$! SLAVE procedure to start  EDT  and do some fancy things on a VT100 terminal.	!$
$!										!$
$! This procedure is assumed to be in the file  SYS$PROCS:SLAVE.COM .   It also	!$
$!	assumes there is a procedure file SYS$PROCS:SLAVEDEF.COM to make symbol	!$
$!	definitions,  an  EDT command file  SYS$PROCS:SLAVINI.EDT  that defines	!$
$!	special keys and macros, and an EDT setup file SYS$PROCS:SETUP.EDT that	!$
$!	contains setup  instructions  for  EDT.  A final assumption is that the	!$
$!	the help library  SYS$HELP:EDTHELP.HLB  has been updated using the file	!$
$!	SYS$PROCS:SLAVE.HLP .							!$
$!										!$
$!	With  these  assumptions  satisfied, these extensions to the editor may	!$
$!	be invoked by accessing  SLAVEDEF  interactively or in your  LOGIN.COM.	!$
$!	Use either								!$
$!										!$
$!		$ @SYS$PROCS:SLAVEDEF BY_FILE	<symbol>			!$
$!	or									!$
$!		$ @SYS$PROCS:SLAVEDEF BY_SYMBOL	<symbol>			!$
$!										!$
$!	BY_FILE causes  SLAVE to store last-edited file information in the file	!$
$!	EDTFILE.DAT  in the current subdirectory;  BY_SYMBOL (the default) lets	!$
$!	SLAVE use the global symbol EDTFILE to store its last-edited file info.	!$
$!	<symbol> is the symbol you wish to use for invoking SLAVE, and defaults	!$
$!	to  EDT  if not specified.   Assume the default symbol was used.   Then	!$
$!	to edit a file, type  EDT [filename]  [/qualifiers].  If no filename is	!$
$!	given, the last-edited file  (BY_FILE or BY_SYMBOL) is assumed.  Do not	!$
$!	use the /command qualifier with this procedure.				!$
$!										!$
$! The procedure looks for a  SETUP.EDT file in the current directory; if it is	!$
$!	not there,  it looks in the user's  home directory  (SYS$LOGIN:)  for a	!$
$!	SETUP.EDT;  if not there either,  it copies one to the  home  directory	!$
$!	from SYS$PROCS:, and uses that copy as a setup file.			!$
$!										!$
$! The procedure also looks in the user's home directory for a SLAVINI.EDT file	!$
$!	to initialize  the session;  if one exists,  it appends it to a copy of	!$
$!	SYS$PROCS:SLAVINI.EDT,  and uses  that for a command file.   Otherwise,	!$
$!	SYS$PROCS:SLAVINI.EDT is used "as is" for a command file.		!$
$!										!$
$! **************************************************************************** !$
$!
$! Initialize a couple of variables
$!
$	SLAVINI := "FALSE"
$	QUALIFIER := ""
$!
$! If P1 is not BY_FILE or BY_SYMBOL, SLAVE is probably being used incorrectly.
$!
$	IF P1 .EQS. "BY_FILE" .OR. P1 .EQS. "BY_SYMBOL" THEN GOTO OK
$	WRITE SYS$OUTPUT -
	"P1 should be ""BY_FILE"" or ""BY_SYMBOL"", instead of ""''P1'""."
$	EXIT
$OK:
$!
$! Store the current directory for use in the filename.
$!
$	DIRY := 'F$LOGICAL("SYS$DISK")''F$DIRECTORY()'
$!
$! Is a filename specified?
$!
$	IF  P2 .NES. "" THEN GOTO X1
$!
$! If not, is there a previous filename?
$!
$PREV_NAME:
$	IF P1 .NES. "BY_FILE" THEN GOTO BY_SYMBOL
$!
$! If user is using files to store "last file" information,
$! look in his local EDTFILE.DAT for last edited file.
$!
$	OPEN /READ /ERR=PROMPT FILE EDTFILE.DAT
$	READ FILE LINE
$	CLOSE FILE
$	EDTFILE :== 'LINE'
$!
$BY_SYMBOL:
$!
$! This extra assignment to the local symbol NAM takes care of the case in
$! which EDTFILE has not yet been defined.
$!
$	NAM := 'EDTFILE'
$	IF NAM .EQS. "" THEN GOTO PROMPT
$!
$! See if the last-used file is ok.
$!
$	INQUIRE P2 -
"Last file edited was [1m''EDTFILE'[m
Optional new file name"
$	IF P2 .EQS. "" THEN GOTO X2
$	GOTO X1
$!
$PROMPT:
$! Ask for a filename.
$!
$	INQUIRE/NOPUNC P2 "$_File:	"
$X1:
$!
$! Parse the entry, looking for file specifications and qualifiers.
$!
$	LEN = 'F$LENGTH(P2)'
$!
$! Look for a switch qualifier after the filename
$!
$	SLASH = 'F$LOCATE("/", P2)'
$	IF SLASH .EQ. LEN THEN GOTO NOQUAL
$!
$! Extract the qualifer and the filename from the first parameter.
$!
$	LMS = LEN - SLASH
$	QUALIFIER := 'QUALIFIER''F$EXTRACT(SLASH, LMS, P2)'
$	P2 := 'F$EXTRACT(0, SLASH, P2)'
$	LEN = SLASH
$!
$! If "/" was not the first character, assume there was a filename specified
$!
$	IF SLASH .NE. 0 THEN GOTO NOQUAL
$!
$! No filename yet...
$!
$! If P3 is null, then only a qualifier with no filename was specified --
$! Go look for the "last-edited" file name.
$!
$	IF P3 .EQS. "" THEN GOTO PREV_NAME
$!
$! If P3 is a tab, then this is the second loop, and still no file name...
$! Give up!
$!
$	IF P3 .EQS. "	" THEN GOTO TOO_MUCH_WORK
$!
$! If I am here, then P2 did not yield a file name, and P3 is nonblank.
$! Put P3 in P2 and go around once more.
$!
$	P2 := 'P3'
$	P3 := "	"	! Set P3 equal to a tab for later reference.
$	GOTO X1
$!
$TOO_MUCH_WORK:
$!
$! Give it one more chance; if P4 is null, then look for "last-edited" filename
$!
$	IF P4 .EQS. "" THEN GOTO PREV_NAME
$!
$	WRITE SYS$OUTPUT "Sorry, SLAVE can't do everything!"
$	WRITE SYS$OUTPUT "Please try again, with no space between qualifiers."
$	EXIT
$!
$NOQUAL:
$!
$! Look for device and/or directory specification in the filename
$!
$	COLON = 'F$LOCATE(":", P2)'
$	BRACKET = 'F$LOCATE("[", P2)'
$	POINT_BRACKET = 'F$LOCATE("<", P2)'
$!
$! If no directory, attach the current default directory to the name.
$!
$	IF (BRACKET .EQ. LEN) .AND. (POINT_BRACKET .EQ. LEN) -
	THEN EDTFILE :== 'DIRY''P2'
$!
$! If device is specified, take the name "as is".
$!
$	IF COLON .NE. LEN THEN EDTFILE :== 'P2'
$!
$! If directory but no device, add the current device to the filename.
$!
$	 IF   (COLON .EQ. LEN) .AND. (BRACKET       .NE. LEN) -
	.OR.  (COLON .EQ. LEN) .AND. (POINT_BRACKET .NE. LEN) -
	THEN  EDTFILE :== 'F$LOGICAL("SYS$DISK")''P2'
$!
$! If lastfile is to be stored in a file, and the edit is not being done 
$! "/read_only", write EDTFILE into EDTFILE.DAT
$!
$	 IF	P1 .NES. "BY_FILE" -
	.OR. 	"''F$EXTRACT(0, 4, QUALIFIER)'" .EQS. "/REA" -
	.OR. 	"''F$EXTRACT(0, 4,        P3)'" .EQS. "/REA" -
	THEN 	GOTO X2
$!
$	OPEN/WRITE FILE EDTFILE.DAT
$	WRITE FILE EDTFILE
$	CLOSE FILE
$	PURGE EDTFILE.DAT
$!
$X2:
$!
$! Exit and error handling (clean up temporary files)
$!
$	ON CONTROL_Y THEN GOTO LVE
$	ON ERROR THEN GOTO ERROR
$!
$!
$! Clear the screen and display the filename (fancy).
$!
$	WRITE SYS$OUTPUT "[2J"
$	WRITE SYS$OUTPUT "[10;7H#6[7mEditing:[m"
$	WRITE SYS$OUTPUT "[12;1H#6[7m''EDTFILE'[m"
$	WRITE SYS$OUTPUT ""
$!
$! Write the filename to a temporary file for reference by EDT.
$!
$	OPEN/WRITE NFILE SYS$LOGIN:FILENM.TMP
$	WRITE NFILE "Editing file: ''EDTFILE'"
$	CLOSE NFILE
$!
$! Look for a setup file in the current directory
$!
$	OPEN /ERR=NO_SUBSETUP DUMDUM SETUP.EDT
$	CLOSE DUMDUM
$	ASSIGN 'DIRY' SETUPDIR
$	WRITE SYS$OUTPUT "
Using a local SETUP.EDT file..."
$	GOTO SLAVINI
$!
$NO_SUBSETUP:
$! No local setup file; look for one in home directory.
$!
$	ASSIGN SYS$LOGIN: SETUPDIR
$	WRITE SYS$OUTPUT "
Using SETUP.EDT file from your root directory..."
$	OPEN /ERR=NO_SETUP DUMDUM SYS$LOGIN:SETUP.EDT
$	CLOSE DUMDUM
$	GOTO SLAVINI
$!
$NO_SETUP:
$! No setup file in home directory; copy one from SYS$PROCS.
$!
$	COPY SYS$PROCS:SETUP.EDT SYS$LOGIN:SETUP.EDT
$!
$SLAVINI:
$! Look for SLAVINI.EDT in home directory.
$!
$	OPEN /ERR=NO_INI DUMDUM SYS$LOGIN:SLAVINI.EDT
$	CLOSE DUMDUM
$	WRITE SYS$OUTPUT "
Using SLAVINI.EDT file from your root directory..."
$	SLAVINI := "TRUE"
$!
$! Concatinate SLAVINI.EDT with the user's SLAVINI.EDT into a temporary SLAVINI.
$!
$	COPY SYS$PROCS:SLAVINI.EDT,SYS$LOGIN:SLAVINI.EDT SYS$LOGIN:SLAVINI.TMP
$!
$! Run EDT with the temporary SLAVINI plus user-specified qualifiers
$!
$	ASSIGN/USER TT SYS$INPUT:
$	EDIT/EDT/COMMAND=SYS$LOGIN:SLAVINI.TMP 'EDTFILE' 'QUALIFIER' 'P3' 'P4' 'P5'
$	GOTO DONE
$!
$NO_INI:
$! No user SLAVINI file; run EDT using only system SLAVINI.EDT command file.
$!
$	ASSIGN/USER TT SYS$INPUT:
$	EDIT/EDT/COMMAND=SYS$PROCS:SLAVINI.EDT 'EDTFILE' 'QUALIFIER' 'P3' 'P4' 'P5'
$!
$DONE:
$! Disable system messages that may confuse the user
$!
$	SET MESSAGE /NOFAC/NOID/NOSEV/NOTEXT
$!
$! Save only the latest setup file.
$	PURGE SETUPDIR:SETUP.EDT
$!
$ERROR:
$! Clean up assignments and temporary files.
$!
$	DELETE/NOCONFIRM/SINCE=TODAY SYS$LOGIN:FILENM.TMP;0
$	DEASSIGN SETUPDIR
$	IF SLAVINI THEN DELETE/NOCONFIRM SYS$LOGIN:SLAVINI.TMP;0
$	DELETE/NOCONFIRM SYS$LOGIN:XYZXX.EDT;*
$!
$! Re-enable system messages
$!
$	SET MESSAGE /FAC/ID/SEV/TEXT
$!
$! Show all versions of edited file.
$!
$!	WRITE SYS$OUTPUT "[H[J"
$	DIRECTORY/NOHEAD/NOTRAIL/DATE 'EDTFILE'
$	WRITE SYS$OUTPUT ""
$!
$! Recall the verify setting and exit.
$!
$	GOTO EXIT
$!
$LVE:
$! Clean up assignments and temporary files.
$!
$	DELETE/NOCONFIRM/SINCE=TODAY SYS$LOGIN:FILENM.TMP;0
$	DEASSIGN SETUPDIR
$	IF SLAVINI THEN DELETE/NOCONFIRM SYS$LOGIN:SLAVINI.TMP;0
$	DELETE/NOCONFIRM SYS$LOGIN:XYZXX.EDT;*
$!
$! Abnormal exit from EDT: reset keypad to numeric mode, clear screen, 
$! and see if user wants to save journal file.
$!
$	TYPE SYS$INPUT:
  >[H[J
   Now leaving EDT improperly, by CTRL_Y !!
   Look for a journal file ................
$!
$!
$	RIGHT_BRACKET = 'F$LOCATE("]", EDTFILE)'
$	LEN = 'F$LENGTH(EDTFILE)'
$ 	IF RIGHT_BRACKET .NE. LEN THEN GOTO LAB1
$!
$	RIGHT_BRACKET = 'F$LOCATE(">", EDTFILE)'
$ 	IF RIGHT_BRACKET .NE. LEN THEN GOTO LAB1
$	RIGHT_BRACKET = -1
$!
$	COLON = 'F$LOCATE(":", EDTFILE)'
$	IF COLON .EQ. LEN THEN GOTO LAB1
$	RIGHT_BRACKET = COLON
$!
$LAB1:	RIGHT_BRACKET = RIGHT_BRACKET + 1
$	DIRY := 'F$EXTRACT(0, RIGHT_BRACKET, EDTFILE)'
$	LEN = LEN - RIGHT_BRACKET
$	NAM := 'F$EXTRACT(RIGHT_BRACKET, LEN, EDTFILE)'
$	DOT = 'F$LOCATE(".", NAM)'
$	NAME_NOEXT := 'F$EXTRACT(0, DOT, NAM)'
$!
$	DIRECTORY/DATE 'DIRY''NAME_NOEXT'.JOU;0
$!
$	INQUIRE/NOPUN FLAG "[BDo you want to save the journal file? "
$!
$	IF .NOT. FLAG THEN DELETE/NOCONFIRM/LOG 'DIRY''NAME_NOEXT'.JOU;0
$!
$! Recall the verify setting and exit.
$!
$EXIT:
$	IF SAV_VER THEN SET VERIFY
