	INTEGER FUNCTION DDOT (COMSTR, COM_LEN, OLDDIR, NEWDISK, DISK_LEN,
	1	NEWDIR)

C  This function is part of the DEFAULT package.

C	Author:	Joseph L. BINGHAM
C		ManTech International
C		2320 Mill Road
C		Alexandria, VA 22314
C		(703) 838-5600

C  This function determines the desired directory when the argument
C    begins with a dot or a letter or a digit according to the following
C    rules:

C      1.  Each leading dot indicates a level in the current directory
C          tree.

C      2.  A string of letters, digits, periods, dashes and dots represents
C          a directory or subdirectory name or the name of a subdirectory
C          attached to the directory indicated by the leading periods.

C      3.  If there are no leading periods in the portion of the command
C          used, an attempt is made to perform a logical name translation
C          on the string.

C  Parameters are as follows:

C      COMSTR - CHARACTER*(*) - input/output.  This is the input command
C         string.  The unused portion of the command string, if any, is
C         returned in COMSTR.  NOTE:  It is assumed that characters in
C         COMSTR are uppercase.  It is also assumed that there is a
C         trailing blank in COMSTR.

C      COM_LEN - I*4 - input/output.  The length of the command string.  It is
C         updated prior to return with the actual length of the remaining part
C         of the command string.

C      OLDDIR - CHARACTER*(*) - input.  This string indicates the current
C         position in the directory tree.

C      NEWDISK - CHARACTER*(*) - output.  The name of the disk to which
C         default is to be set if the part of the command string interpreted
C         specifies one.

C      DISK_LEN - I*4 output.  The length of the specified disk name.  Zero
C         if none is specified.

C      NEWDIR - CHARACTER*(*) - output.  This string is the name of the
C         desired directory as indicated by the portion of COMSTR used
C         during this call to DDOTS.

C  Return status meanings:

C      1 - Normal return.

C      3 - Subdirectory depth indicated by dots exceeds the current
C          subdirectory depth.  Current subdirectory depth is assumed.

C  Modifications:

C	82MAY03 - Modified to properly treat ',' as a delimiter.  Prior to
C		the modification name,N was not handled correctly.

	IMPLICIT INTEGER (A-Z)

	CHARACTER*(*) COMSTR, OLDDIR, NEWDISK, NEWDIR
	CHARACTER*1 BRACKET
	INTEGER BPTR, COMPTR, COMPTR2, DOTPTR, COM_LEN, DISK_LEN, DOTS,
	1 STATUS

C Find out wheather the directory name is in square or angle brackets and
C   the location of the closing bracket.

	BPTR = INDEX (OLDDIR, ']')
	IF (BPTR .NE. 0) THEN

		BRACKET = ']'

	ELSE

		BRACKET = '>'
		BPTR = INDEX (OLDDIR, '>')

	ENDIF

	COMPTR = 1
	STATUS = 1
	DISK_LEN = 0

C Determine the number of leading dots.

	DO WHILE (COMSTR (COMPTR:COMPTR) .EQ. '.')

		COMPTR = COMPTR + 1

	ENDDO

	DOTS = COMPTR - 1
	COMPTR2 = COMPTR

C Handle the condition of no leading dots.

	IF (DOTS .EQ. 0) THEN

		DO WHILE (COMSTR (COMPTR2:COMPTR2) .GE. '0' .AND.
	1		COMSTR (COMPTR2:COMPTR2) .LE. '9' .OR.
	2		COMSTR (COMPTR2:COMPTR2) .GE. 'A' .AND.
	3		COMSTR (COMPTR2:COMPTR2) .LE. 'Z' .OR.
	3		COMSTR (COMPTR2:COMPTR2) .EQ. '.' .OR.
	8		COMSTR (COMPTR2:COMPTR2) .EQ. '-')

			COMPTR2 = COMPTR2 + 1

		ENDDO

		IF (COMSTR (COMPTR2:COMPTR2) .EQ. ':') THEN

			NEWDISK = COMSTR (COMPTR:COMPTR2)
			DISK_LEN = COMPTR2 - COMPTR + 1
			COMPTR = COMPTR2 + 1

		ENDIF

		IF (COMSTR (COMPTR:COMPTR) .EQ. '[') THEN

			COMPTR2 = INDEX (COMSTR (COMPTR:), ']') + COMPTR - 1
			NEWDIR = COMSTR (COMPTR: COMPTR2)

		ELSE IF (COMSTR (COMPTR:COMPTR) .EQ. '<') THEN

			COMPTR2 = INDEX (COMSTR (COMPTR:), '>') + COMPTR - 1
			NEWDIR = COMSTR (COMPTR: COMPTR2)

		ELSE IF (COMSTR (COMPTR2:COMPTR2) .EQ. ':' .AND.
	1		COMSTR (COMPTR:COMPTR) .EQ. ' ') THEN

			COMPTR2 = INDEX (COMSTR (COMPTR:), ' ') + COMPTR - 1
			NEWDIR = ' '

		ELSE

			IF (COMSTR (COMPTR2:COMPTR2) .EQ. ',') THEN

				COMSTR = COMSTR (:COMPTR2-1) // ' ' // 
	1				COMSTR (COMPTR2:)
				COM_LEN = COM_LEN + 1

			ENDIF
C Here is where we attempt to do a logical name translation.
			IF (SYS$TRNLOG ('DIR$' // COMSTR (COMPTR:COMPTR2-1)
	1			, , NEWDIR, , ,) .EQ. 1) THEN

				DISK_LEN = INDEX (NEWDIR, ':')
				IF (DISK_LEN.NE.0) NEWDISK = NEWDIR(:DISK_LEN)

			ELSE

				NEWDIR='[' // COMSTR (COMPTR: COMPTR2-1) // ']'

			ENDIF

		ENDIF

	ELSE

C There are leading dots.  Get the subdirectory name if one is specified.

		DOTPTR = 0

		DO WHILE (COMSTR (COMPTR2:COMPTR2) .GE. '0' .AND.
	1		COMSTR (COMPTR2:COMPTR2) .LE. '9' .OR.
	2		COMSTR (COMPTR2:COMPTR2) .GE. 'A' .AND.
	3		COMSTR (COMPTR2:COMPTR2) .LE. 'Z' .OR.
	3		COMSTR (COMPTR2:COMPTR2) .EQ. '.' .OR.
	8		COMSTR (COMPTR2:COMPTR2) .EQ. '-')

			COMPTR2 = COMPTR2 + 1

		ENDDO

C Step to the desired position in OLDDIR.

		DO IDOTS = 1, DOTS

			NEXTDOT = INDEX (OLDDIR (DOTPTR+1:), '.')
			DOTPTR = NEXTDOT + DOTPTR

			IF (NEXTDOT .EQ. 0) THEN

				IF (IDOTS .LT. DOTS) STATUS = 3
				DOTPTR = BPTR

			ENDIF

		ENDDO

C Put the name together.

		COMPTR2 = COMPTR2 - 1

		IF (COMPTR2 .GE. COMPTR) THEN

			NEWDIR = OLDDIR(:DOTPTR-1) // '.' //
	1			COMSTR(COMPTR:COMPTR2) // BRACKET

		ELSE

			NEWDIR = OLDDIR (:DOTPTR-1) // BRACKET

		ENDIF

	ENDIF

	COMSTR = COMSTR (COMPTR2+1:)
	COM_LEN = COM_LEN - COMPTR2
	DDOT = STATUS

	RETURN

	END
	FUNCTION DNEXT (OLDDIR, NEWDIR)

C This function is part of the DEFAULT package.

C Author:	Joseph L. BINGHAM
C		ManTech International
C		2320 Mill Road
C		Alexandria, VA 22314
C		(703) 838-5600

C This function finds the next directory in the directory tree.  It
C   requires a subroutine DIR to look up the directory files contained
C   in a given directory and a function UPONE which returns the name
C   of the next higher level in the directory tree.  UPONE stops the
C   program if it is called with the 000000 directory as an arqument or
C   if it cannot find the trailing bracket delimiting the directory name.

C Parameters are as follows:

C   OLDDIR - CHARACTER*(*) - input.  This string indicates the current
C     position in the directory tree.

C   NEWDIR - CHARACTER*(*) - output.  This is the name of the next directory
C     in the tree in prefix order.

	IMPLICIT INTEGER (A-Z)
	CHARACTER *(*) OLDDIR, NEWDIR
	CHARACTER*63 SUBDIRS (150), NODE, ABOVENODE

	DNEXT = 1
	NODE = OLDDIR (INDEX (OLDDIR, ':') + 1:)
	ABOVENODE = NODE

	DO WHILE (.TRUE.)

		SUBDIRS (1)(1:1) = ' '
C		Get the list of subdirectories.
		CALL DIR (ABOVENODE, SUBDIRS)
		I = 1

C		Step to the next subdirectory if there is one.
		DO WHILE((NODE .GE. SUBDIRS (I) (INDEX (SUBDIRS(I), ':')+1:)
	2		.OR. '[000000]000000.DIR;1' .GE.
	3		SUBDIRS (I) (INDEX (SUBDIRS (I), ':')+1:))
	1		.AND.SUBDIRS (I)(1:1).NE. ' ')

			I = I + 1

		ENDDO

C		Handle the case of going to a first line directory.
		IF (INDEX (NODE, '[000000]') .NE. 0) THEN

			IF (SUBDIRS (I)(1:1) .EQ. ' ') THEN

				I = 1

			ELSE

				RHB = INDEX (SUBDIRS (I), ']')
				DOT = INDEX (SUBDIRS (I)(RHB:),'.') + RHB
				NEWDIR='['//SUBDIRS (I) (RHB+1:DOT-2)//']'
				RETURN

			ENDIF

		ENDIF

C		Handle the case of going to a subdirectory.
		IF (SUBDIRS (I) (1:1) .NE. ' ') THEN

C Change, e.g., DRA0:[SYSPROG.CLI]TEST.DIR;1 to DRA0:[SYSPROG.CLI.TEST]
			RHB = INDEX (SUBDIRS (I), ']')
			DOT = INDEX (SUBDIRS (I)(RHB:), '.') + RHB
			NEWDIR = SUBDIRS (I) (:RHB-1) // '.' //
	1			SUBDIRS (I)(RHB+1:DOT-2) // ']'
			RETURN

		ENDIF

C No subdirectory found. Check the next higher level.

C UPONE transfers ABOVENODE to NODE and changes it from directory syntax
C   (DRA0:[SYSPROG.CLI]) to file syntax in the next higher directory
C   (DRA0:[SYSPROG]CLI.DIR;1).  It also moves ABOVENODE up one directory.

		DNEXT = UPONE (NODE, ABOVENODE)
		IF (.NOT. DNEXT) RETURN

	ENDDO

	END
	FUNCTION UPONE (NODE, ABOVENODE)

C This function is part of the DEFAULT package.

C Author:	Joseph L. BINGHAM
C		ManTech International
C		2320 Mill Road
C		Alexandria, VA 22314
C		(703) 838-5600

C Using ABOVENODE as the current directory name (in directory name form)
C   it sets NODE to the name of that directory file and ABOVENODE to the
C   name of the next higher directory, the one in which that file resides.
C   Error if called with [000000] in ABOVENODE.

	IMPLICIT INTEGER (A-Z)
	CHARACTER*(*) NODE, ABOVENODE
	CHARACTER*1 BRACKET

	UPONE = 1

	IF (INDEX (ABOVENODE, '[000000]') .NE. 0) THEN

		UPONE = 0
		RETURN

	ENDIF

	RHB = LEN (ABOVENODE)

	DO WHILE (ABOVENODE (RHB:RHB) .EQ. ' ' .AND. RHB .GT. 0)

		RHB = RHB - 1

	ENDDO

	IF (ABOVENODE (RHB:RHB) .EQ. ']') THEN

		BRACKET = ']'

	ELSE IF (ABOVENODE (RHB:RHB) .EQ. '>') THEN

		BRACKET = '>'

	ELSE

		TYPE*, 'Error in D command.  UPONE unable to find RH bracket.'
		STOP

	ENDIF

	DOT = RHB

	DO WHILE (ABOVENODE (DOT:DOT) .NE. '.' .AND. DOT .GT. 0)

		IF (DOT .EQ. 1) THEN

			NODE='[000000]'//ABOVENODE(DOT+1:RHB-1)//'.DIR;1'
			ABOVENODE = '[000000]'
			RETURN

		ENDIF

		DOT = DOT - 1

	ENDDO

	NODE = ABOVENODE (:DOT-1) // BRACKET // ABOVENODE (DOT+1:RHB-1)
	1	// '.DIR;1'
	ABOVENODE = NODE (:DOT)

	RETURN

	END
